@@ -1,34 +1,41 @@
+from __future__ import unicode_literals
 import datetime
-from StringIO import StringIO
+import re
 import django
 from django.conf import settings
 from django.core.exceptions import ImproperlyConfigured
-from django.core.serializers import json
-try:
-    import json as simplejson
-except ImportError: # < Python 2.6
-    from django.utils import simplejson
-from django.utils.encoding import force_unicode
+from django.utils import six
+from django.utils.encoding import force_text, smart_bytes
+from django.core.serializers import json as djangojson
+
 from tastypie.bundle import Bundle
 from tastypie.exceptions import BadRequest, UnsupportedFormat
 from tastypie.utils import format_datetime, format_date, format_time, make_naive
+
 try:
     import defusedxml.lxml as lxml
     from defusedxml.common import DefusedXmlException
     from defusedxml.lxml import parse as parse_xml
-    from lxml.etree import Element, tostring, LxmlError
+    from lxml.etree import Element, tostring, LxmlError, XMLParser
 except ImportError:
     lxml = None
+
 try:
     import yaml
     from django.core.serializers import pyyaml
 except ImportError:
     yaml = None
+
 try:
     import biplist
 except ImportError:
     biplist = None
 
+import json
+
+
+XML_ENCODING = re.compile('<\?xml.*?\?>', re.IGNORECASE)
+
 
 # Ugh & blah.
 # So doing a regular dump is generally fine, since Tastypie doesn't usually
@@ -140,7 +147,7 @@ class Serializer(object):
         if self.datetime_formatting == 'iso-8601-strict':
             # Remove microseconds to strictly adhere to iso-8601
             data = data - datetime.timedelta(microseconds = data.microsecond)
-            
+
         return data.isoformat()
 
     def format_date(self, data):
@@ -174,12 +181,14 @@ class Serializer(object):
 
         return data.isoformat()
 
-    def serialize(self, bundle, format='application/json', options={}):
+    def serialize(self, bundle, format='application/json', options=None):
         """
         Given some data and a format, calls the correct method to serialize
         the data and returns the result.
         """
         desired_format = None
+        if options is None:
+            options = {}
 
         for short_format, long_format in self.content_types.items():
             if format == long_format:
@@ -211,6 +220,9 @@ class Serializer(object):
         if desired_format is None:
             raise UnsupportedFormat("The format indicated '%s' had no available deserialization method. Please check your ``formats`` and ``content_types`` on your Serializer." % format)
 
+        if isinstance(content, six.binary_type):
+            content = force_text(content)
+
         deserialized = getattr(self, "from_%s" % desired_format)(content)
         return deserialized
 
@@ -225,9 +237,9 @@ class Serializer(object):
         if isinstance(data, (list, tuple)):
             return [self.to_simple(item, options) for item in data]
         if isinstance(data, dict):
-            return dict((key, self.to_simple(val, options)) for (key, val) in data.iteritems())
+            return dict((key, self.to_simple(val, options)) for (key, val) in data.items())
         elif isinstance(data, Bundle):
-            return dict((key, self.to_simple(val, options)) for (key, val) in data.data.iteritems())
+            return dict((key, self.to_simple(val, options)) for (key, val) in data.data.items())
         elif hasattr(data, 'dehydrated_type'):
             if getattr(data, 'dehydrated_type', None) == 'related' and data.is_m2m == False:
                 if data.full:
@@ -249,12 +261,12 @@ class Serializer(object):
             return self.format_time(data)
         elif isinstance(data, bool):
             return data
-        elif type(data) in (long, int, float):
+        elif isinstance(data, (six.integer_types, float)):
             return data
         elif data is None:
             return None
         else:
-            return force_unicode(data)
+            return force_text(data)
 
     def to_etree(self, data, options=None, name=None, depth=0):
         """
@@ -270,18 +282,21 @@ class Serializer(object):
                 element = Element('objects')
             for item in data:
                 element.append(self.to_etree(item, options, depth=depth+1))
+                element[:] = sorted(element, key=lambda x: x.tag)
         elif isinstance(data, dict):
             if depth == 0:
                 element = Element(name or 'response')
             else:
                 element = Element(name or 'object')
                 element.set('type', 'hash')
-            for (key, value) in data.iteritems():
+            for (key, value) in data.items():
                 element.append(self.to_etree(value, options, name=key, depth=depth+1))
+                element[:] = sorted(element, key=lambda x: x.tag)
         elif isinstance(data, Bundle):
             element = Element(name or 'object')
             for field_name, field_object in data.data.items():
                 element.append(self.to_etree(field_object, options, name=field_name, depth=depth+1))
+                element[:] = sorted(element, key=lambda x: x.tag)
         elif hasattr(data, 'dehydrated_type'):
             if getattr(data, 'dehydrated_type', None) == 'related' and data.is_m2m == False:
                 if data.full:
@@ -308,10 +323,10 @@ class Serializer(object):
                 element.set('type', get_type_string(simple_data))
 
             if data_type != 'null':
-                if isinstance(simple_data, unicode):
+                if isinstance(simple_data, six.text_type):
                     element.text = simple_data
                 else:
-                    element.text = force_unicode(simple_data)
+                    element.text = force_text(simple_data)
 
         return element
 
@@ -356,16 +371,16 @@ class Serializer(object):
         options = options or {}
         data = self.to_simple(data, options)
 
-        if django.get_version() >= '1.5':
-            return json.json.dumps(data, cls=json.DjangoJSONEncoder, sort_keys=True, ensure_ascii=False)
-        else:
-            return simplejson.dumps(data, cls=json.DjangoJSONEncoder, sort_keys=True, ensure_ascii=False)
+        return djangojson.json.dumps(data, cls=djangojson.DjangoJSONEncoder, sort_keys=True, ensure_ascii=False)
 
     def from_json(self, content):
         """
         Given some JSON data, returns a Python dictionary of the decoded data.
         """
-        return simplejson.loads(content)
+        try:
+            return json.loads(content)
+        except ValueError:
+            raise BadRequest
 
     def to_jsonp(self, data, options=None):
         """
@@ -405,10 +420,16 @@ class Serializer(object):
             raise ImproperlyConfigured("Usage of the XML aspects requires lxml and defusedxml.")
 
         try:
-            parsed = parse_xml(StringIO(content), forbid_dtd=forbid_dtd,
-                               forbid_entities=forbid_entities)
+            # Stripping the encoding declaration. Because lxml.
+            # See http://lxml.de/parsing.html, "Python unicode strings".
+            content = XML_ENCODING.sub('', content)
+            parsed = parse_xml(
+                six.StringIO(content),
+                forbid_dtd=forbid_dtd,
+                forbid_entities=forbid_entities
+            )
         except (LxmlError, DefusedXmlException):
-            raise BadRequest
+            raise BadRequest()
 
         return self.from_etree(parsed.getroot())
 
@@ -450,6 +471,9 @@ class Serializer(object):
         if biplist is None:
             raise ImproperlyConfigured("Usage of the plist aspects requires biplist.")
 
+        if isinstance(content, six.text_type):
+            content = smart_bytes(content)
+
         return biplist.readPlistFromString(content)
 
     def to_html(self, data, options=None):
@@ -479,7 +503,7 @@ def get_type_string(data):
     """
     data_type = type(data)
 
-    if data_type in (int, long):
+    if data_type in six.integer_types:
         return 'integer'
     elif data_type == float:
         return 'float'
@@ -491,5 +515,5 @@ def get_type_string(data):
         return 'hash'
     elif data is None:
         return 'null'
-    elif isinstance(data, basestring):
+    elif isinstance(data, six.string_types):
         return 'string'
